@@ -18,5 +18,5 @@ class TimeoutException(Exception):
 
 class MovesExceededException(Exception):
     def __init__(self) -> None:
-        self.message = "Game exceeeded limit for moves without any player winning. Declared as a draw"
+        self.message = "Game exceeeded limit for total no. of moves without any player winning. Declared as a draw"
         super().__init__(self.message)
