@@ -1,6 +1,5 @@
 from email import message
 from typing import Any, List, Tuple
-import json
 import logging
 
 import requests
@@ -8,10 +7,10 @@ from requests import Timeout
 from colorama import Fore, Style
 from pydantic import ValidationError
 
-from .schema import Piece, Move, Player, InvalidMovesMessage, BotResponse, PlayerMove
-from .custom_exceptions import InvalidMovesException, TimeoutException, MovesExceededException
-from .constants import MAX_MOVES_ALLOWED
-from .logger_utils import get_logger
+from schema import Piece, Move, Player, InvalidMovesMessage, BotResponse, PlayerMove
+from custom_exceptions import InvalidMovesException, TimeoutException, MovesExceededException
+from constants import MAX_MOVES_ALLOWED
+from logger_utils import get_logger, game_visualizer
 
 
 class Simulator:
@@ -22,24 +21,27 @@ class Simulator:
         self.BLUE_PIECES = [Piece.BLUE_BOMBER, Piece.BLUE_STINGER]
         self.move_timeout = move_timeout
         self.logger = get_logger(__name__, level=logging_level)
+        self.visualizer = game_visualizer(level=logging_level)
         self.verbose = verbose
 
     def print_board(self):
         """
             Print Board state while testing bots in local development
         """
-        if self.verbose:
-            print(Fore.BLACK + Style.BRIGHT + "  0 1 2 3 4")
-            for index, row in enumerate(self.board):
-                print(Fore.BLACK + Style.BRIGHT + str(index) + " ", end="")
-                for cell in row:
-                    if cell in self.RED_PIECES:
-                        print(Fore.RED + cell + " ", end="")
-                    elif cell in self.BLUE_PIECES:
-                        print(Fore.BLUE + cell + " ", end="")
-                    else:
-                        print(Fore.WHITE + cell + " ", end="")
-                print(Fore.RESET + Style.RESET_ALL)
+        self.visualizer.info(Fore.BLACK + Style.BRIGHT + "  0 1 2 3 4")
+        for index, row in enumerate(self.board):
+            row_message = Fore.BLACK + Style.BRIGHT + str(index) + " "
+            for cell in row:
+                if cell in self.RED_PIECES:
+                    row_message += Fore.RED + cell + " "
+                elif cell in self.BLUE_PIECES:
+                    row_message += Fore.BLUE + cell + " "
+                else:
+                    row_message += Fore.WHITE + cell + " "
+            # row_message = Fore.RESET + Style.RESET_ALL
+            self.visualizer.info(row_message)
+        self.visualizer.info(Fore.BLACK + Style.DIM + "==================================================")
+        self.visualizer.info(Fore.RESET + Style.RESET_ALL)
 
     @staticmethod
     def validate_direction(move: Move, player: Player):
@@ -138,15 +140,15 @@ class Simulator:
         move : Move
         player : Player
         """
-        self.logger.info(f"Validating Player {player} Move")
-        if self.verbose:
-            if player == Player.BLUE:
-                color = Fore.BLUE
-            else:
-                color = Fore.RED
-            print(color + Style.BRIGHT + f"Player {player} move")
-            print(color + Style.BRIGHT + str(move))
-            print(Fore.RESET + Style.RESET_ALL)
+        self.logger.debug(f"Validating Player {player} Move")
+
+        if player == Player.BLUE:
+            color = Fore.BLUE
+        else:
+            color = Fore.RED
+        self.visualizer.info(color + Style.BRIGHT + f"Player {player} move")
+        self.visualizer.info(color + Style.BRIGHT + str(move))
+        self.visualizer.info(Fore.RESET + Style.RESET_ALL)
         self.validate_direction(move, player)
         self.validate_endpoints(move, player)
         piece = self.board[move.src.row][move.src.col]
@@ -182,7 +184,7 @@ class Simulator:
 
     def run(self, bot_urls: List[str], bot_data: List[Any], player_moves: List[PlayerMove]):
         player = Player.BLUE
-        self.logger.info("STARTING GAME")
+        self.logger.debug("Starting game simulation")
         self.print_board()
         while len(player_moves) < MAX_MOVES_ALLOWED:
             bot_input = {"board": self.board, "data": bot_data[player], "player": player}
@@ -203,8 +205,7 @@ class Simulator:
                 self.print_board()
                 game_over, winner = self.check_if_game_over()
                 if game_over:
-                    if self.verbose:
-                        print(f"Player {player} wins")
+                    self.logger.info(f"Player {player} wins")
                     return winner
                 if player == Player.BLUE:
                     player = Player.RED
@@ -220,3 +221,21 @@ class Simulator:
                 self.logger.error(f"Invalid response received from player: {player}. Message: ", str(ve))
                 raise InvalidMovesException(reason=f"Incorrect schema", player=player)
         raise MovesExceededException()
+
+    def replay(self, player_moves: List[PlayerMove]):
+        self.print_board()
+        for player_move in player_moves:
+            if player_move is None:
+                return
+            player = player_move.player
+            move = player_move.move
+            try:
+                self.validate_move(move=move, player=player)
+                self.make_move(player=player, move=move)
+                self.print_board()
+                winner = self.check_if_game_over()
+                if winner[0]:
+                    self.logger.info(f"Player {winner[1]} wins!")
+            except InvalidMovesException as ex:
+                self.logger.error(f"Invalid response received from player: {player}. Message: {ex.message}")
+                raise InvalidMovesException(reason=ex.reason, player=player)  # append player info to exception
